import java.util.BitSet;

/**
* Class used to represent the motif variants calculated during runtime
* @author Vetle Valebjrg
* @version 1.0
* @see CompositeMotif, RealCompositeMotif, FictiveCompositeMotif, SingleMotif
*/
public class MotifVariant{

	private double significance;
	private int support;
	private BitSet bitset;
	private boolean dummy = false;
	private static int idcounter=0;
	private int id;
	
	/**
	* Default Constructor
	*/
    public MotifVariant(){
    	bitset = new BitSet();
    	id = idcounter+1;
    	idcounter++;
    }
    
    /**
    * Makes the motif variant a dummy variant which is used for
    * calculataions outside the top motif variant border
    */
    public void makeDummy(){
    	significance = 0.0;
    	bitset = new BitSet(Mofn.nr_of_seq+1);
    	dummy = true;
    	bitset.set(0,Mofn.nr_of_seq+1);
    	support = bitset.cardinality(); 
    }
    
    /**
    * Makes the motif variant a first variant which is used for 
    * calcualtions in the beginning of every motif variant column
    */
    public void makeFirst(){
		significance = 1.0;
		bitset = new BitSet(Mofn.nr_of_seq+1);
    	//set all true
    	bitset.set(0,Mofn.nr_of_seq+1);
    	support = bitset.cardinality(); 
	}
     
    /**
    * Returns the unique ID for this motif variant
    * @return ID of the motif variant
    */
  	public int getId(){
  		return id;
  	}
  	
  	/**
  	* Returns true if motif variants is a dummy variant
  	* @return True if isDummy true, otherwise false.
  	*/
	public boolean isDummy(){
		return dummy;
	}
	
	/**
	* Method used to calculate a new motif variant based on predecessors and new single motif
	* @param mv1 Predecessor Motif variant diagonal
	* @param mv2 Predecessor Motif variant horizontal
	* @param sm1 New single motif
	* @return The new motifvariant
	* @see SingleMotif
	*/
    public MotifVariant calc_mn(MotifVariant mv1, MotifVariant mv2, SingleMotif sm1){    	
    	//sign_c = P(a&b | d) = P( !( !(a&b) & !d) )
    	//sign_c = 1 - (1 - sign_a * sign_b) * (1 - sign_d)
    	significance  = 1-(1 - mv1.getSign()*sm1.getSign())*(1-mv2.getSign());
  /*  	
    	Mofn.add_output("MV1: " + mv1.getSign() + " id: " + mv1.getId());
    	Mofn.add_output("SM1: " + sm1.getSign());
    	Mofn.add_output("MV2: " + mv2.getSign() + " id: " + mv2.getId());
    	
    	Mofn.add_output("MV1b: " + mv1.getBitSet().toString());
    	Mofn.add_output("SM1b: " + sm1.getBitSet().toString());
    	Mofn.add_output("MV2b: " + mv2.getBitSet().toString());
  */  	
    	//if(Double.isInfinite(log(significance))){
		//	significance = Math.min(log(mv1.getSign())+sm1.getSign(),mv2.getSign());
		//}
    	
    	//bitstr_c = (bitstr_a AND bitstr_b) OR bitstr_d
    	bitset.clear();
    	
    	bitset = (BitSet)mv1.getBitSet().clone();
    	bitset.and(sm1.getBitSet());
    	
    	if(!mv2.isDummy()){
    		bitset.or(mv2.getBitSet());
    	}
    	support = bitset.cardinality();
    	
 //   	Mofn.add_output("RES: " + bitset.toString() + "  SIGN:" + getSign() + "  SUPP:" + getSupport() + " id: " + this.getId());
 //   	Mofn.add_output("");
    	
    	return this;
    }
    
    /**
    * Returns significance calculated motif variant
    * @return Significance
    */
    public double getSign(){
    	return significance;
    }
    
    /**
    * Returns support calculated in motif variant
    * @return The support
    */
    public int getSupport(){
    	return support;
    }
	
	/**
	* Returns bitset calculated in motif variant
	* @return Bitset
	*/
	public BitSet getBitSet(){
		return bitset;
	}

	/*
    public double log(double innum){
    	return Math.log(innum)/Math.log(2);
    }
	*/
}

