import java.util.BitSet;

/**
 * This class contains the structure for motifs used by Mofn (IUPAC/PWM)
 * @author Vetle Valebjrg
 * @version 2.0
 * @see InputReader, Mofn
 */
public class SingleMotif{
	private String name = "MotifX";
	private double significance;
	private String pattern;
	private	int nrOfHits;
	private int nrOfSeq;
	private int[] listOfSeq;
	private BitSet mBitList;
	
	//spesielt for matriser
	private double[][] matrix;
	private double[] rowsum;
	private int rows;
	private int cols;
	
	/**
	* Default constructor
	*/
	public SingleMotif(){
	}
	
	/**
	* Constructor taking all the values from the InputReader as input
	* @param mHits Total hits as int., bitlist as BitSet
	* @param mSeq Hits as distinct sequences.
	* @param mSign Significance.
	* @param mMotif Pattern if(iupac, name if pwm).
	* @param mHitList list of hits and sequences, alternating seqnr and posnr.
	* @param mbList Bitlist represention of hits, true false for 0 -> seq size.
	* @see BitSet
	*/
	public SingleMotif(int mHits, int mSeq, double mSign, String mMotif, int[] mHitList,BitSet mbList){
		pattern = mMotif;
		significance = mSign;
		nrOfHits = mHits;
		nrOfSeq = mSeq;
		listOfSeq = mHitList;
		mBitList = mbList;

	}
	
	/**
	* Constructor used by the prune-method
	* Constructs best motif possible based on prev. motifs
	* @param  mhits Total hits.
	* @param mSeq Distinct hit in seq. 
	* @param mSign Significance
	*/
	public SingleMotif(int mHits, int mSeq, double mSign){
		pattern = "";
		significance = mSign;
		nrOfHits = mHits;
		nrOfSeq = mSeq;
		listOfSeq = new int[0];
		
		mBitList = new BitSet(Mofn.nr_of_seq+1);
    	//set all true
    	mBitList.set(0,Mofn.nr_of_seq+1);
    	
		//mBitList = new BitSet(mSeq+1); //in this case, since max
		//mBitList.set(0, mSeq);
	}

	/**
	* Returns pattern representation as string.
	* @return pattern as string
	*/
	public String getPattern(){
		return pattern;
	}
	
	/**
	* Returns matrix used to represent PWMs
	* @param Matrix values in PWM.
	*/
	public double[][] getMatrix(){
		return matrix;
	}
	
	/**
	* Returns a vector with the sum for each row in the PWM matrix
	* @param Vector with sum for each matrix row.
	*/
	public double[] getRowSum(){
		return rowsum;
	}
	
	/**
	* Returns the number of rows in a PWM, length of the single motif
	* @param Number of rows in PWM matrix.
	*/
	public int getRows(){
		return rows;
	}
	
	/**
	* Returns the number of columns in a PWM, always 4 though
	* @param Number of columns in PWM matrix.
	*/
	public int getCols(){
		return cols;
	}
	
	/**
	* Returns the motif name. An ID is used if no name provided.
	* @param Motif name or ID created.
	*/
	public String getName(){
		return name;
	}
	
	/**
	* Returns the motif significance based on the model used
	* @return Significance
	*/
	public double getSign(){
		return significance;
	}
	
	/**
	* Returns the BitSet used to represent sequence hits
	* @return Bitset representation of the support.
	*/
	public BitSet getBitSet(){
		return mBitList;
	}
	
	/**
	* Returns the hit data provied or found as a list
	* @return List of alternating seqnr and hitpos
	*/
	public int[] getHitData(){
		return listOfSeq;
	}
	
	/**
	* Returns the support found, bitset.cardinality
	* @return Support
	*/
	public int getSupp(){
		return nrOfSeq;
	}
	
	/**
	* Sets the significance based on new calculations
	* @param newSign New significance
	*/
	public void setSign(double newSign){
		significance = newSign;
	}
	
	/**
	* Sets the motif name found
	* @param newName New name
	*/
	public void setName(String newName){
		name = newName;
	}
	
	/**
	* Sets the PWM matrix
	* @param m PWM matrix
	* @param r Rows
	* @param c columns
	* @param sum Vector with row sums
	*/
	public void setMatrix(double[][] m, int r, int c, double[] sum){
		matrix = m;
		rows = r;
		cols = c;
		rowsum = sum;
	}
	
	/**
	* Sets hit data found
	* @param bs New bitset representation of hits in sequences
	* @param hitList Hit data with sequence number and position number for hits
	* @param htot Hits total 
	* @param hseq Distinct number of sequences
	*/
	public void setHitInfo(BitSet bs, int[] hitList, int htot, int hseq){
		nrOfHits = htot;
		nrOfSeq = hseq;
		mBitList = (BitSet)bs.clone();
		listOfSeq = hitList;
	}
		 
		
}
